@@ -44,13 +44,15 @@ from graphistry.pygraphistry import (  # noqa: E402, F401
     ArrowUploader,
     ArrowFileUploader,
     PyGraphistry,
-    from_igraph
+    from_igraph,
+    from_cugraph
 )
 
 from graphistry.compute import (
     n, e_forward, e_reverse, e_undirected
 )
 
+from graphistry.Engine import Engine
 
 from ._version import get_versions
 
